@@ -185,7 +185,7 @@ skipped_modules = {
     'hash': None,
     'list': None,
     'threads': None,
-    'xpointer': None,
+#    'xpointer': None,
 }
 skipped_types = {
     'int *': "usually a return type",
@@ -271,6 +271,11 @@ py_types = {
     'xmlOutputBufferPtr': ('O', "outputBuffer", "xmlOutputBufferPtr", "xmlOutputBufferPtr"),
     'xmlParserInputBufferPtr': ('O', "inputBuffer", "xmlParserInputBufferPtr", "xmlParserInputBufferPtr"),
     'xmlRegexpPtr': ('O', "xmlReg", "xmlRegexpPtr", "xmlRegexpPtr"),
+    'xmlTextReaderLocatorPtr': ('O', "xmlTextReaderLocator", "xmlTextReaderLocatorPtr", "xmlTextReaderLocatorPtr"),
+    'xmlTextReaderPtr': ('O', "xmlTextReader", "xmlTextReaderPtr", "xmlTextReaderPtr"),
+    'xmlRelaxNGPtr': ('O', "relaxNgSchema", "xmlRelaxNGPtr", "xmlRelaxNGPtr"),
+    'xmlRelaxNGParserCtxtPtr': ('O', "relaxNgParserCtxt", "xmlRelaxNGParserCtxtPtr", "xmlRelaxNGParserCtxtPtr"),
+    'xmlRelaxNGValidCtxtPtr': ('O', "relaxNgValidCtxt", "xmlRelaxNGValidCtxtPtr", "xmlRelaxNGValidCtxtPtr"),
 }
 
 py_return_types = {
@@ -289,8 +294,15 @@ unknown_types = {}
 def skip_function(name):
     if name[0:12] == "xmlXPathWrap":
         return 1
+    if name == "xmlFreeParserCtxt":
+        return 1
+    if name == "xmlFreeTextReader":
+        return 1
 #    if name[0:11] == "xmlXPathNew":
 #        return 1
+    # the next function is defined in libxml.c
+    if name == "xmlRelaxNGFreeValidCtxt":
+        return 1
     return 0
 
 def print_function_wrapper(name, output, export, include):
@@ -353,8 +365,8 @@ def print_function_wrapper(name, output, export, include):
 	    if args[1][1] == "char *" or args[1][1] == "xmlChar *":
 		c_call = "\n    if (%s->%s != NULL) xmlFree(%s->%s);\n" % (
 		                 args[0][0], args[1][0], args[0][0], args[1][0])
-		c_call = c_call + "    %s->%s = xmlStrdup((const xmlChar *)%s);\n" % (args[0][0],
-		                 args[1][0], args[1][0])
+		c_call = c_call + "    %s->%s = (%s)xmlStrdup((const xmlChar *)%s);\n" % (args[0][0],
+		                 args[1][0], args[1][1], args[1][0])
 	    else:
 		c_call = "\n    %s->%s = %s;\n" % (args[0][0], args[1][0],
 						   args[1][0])
@@ -414,6 +426,11 @@ def print_function_wrapper(name, output, export, include):
         include.write("#ifdef LIBXML_REGEXP_ENABLED\n");
         export.write("#ifdef LIBXML_REGEXP_ENABLED\n");
         output.write("#ifdef LIBXML_REGEXP_ENABLED\n");
+    elif file == "xmlschemas" or file == "xmlschemastypes" or \
+         file == "relaxng":
+        include.write("#ifdef LIBXML_SCHEMAS_ENABLED\n");
+        export.write("#ifdef LIBXML_SCHEMAS_ENABLED\n");
+        output.write("#ifdef LIBXML_SCHEMAS_ENABLED\n");
 
     include.write("PyObject * ")
     include.write("libxml_%s(PyObject *self, PyObject *args);\n" % (name));
@@ -477,6 +494,11 @@ def print_function_wrapper(name, output, export, include):
         include.write("#endif /* LIBXML_REGEXP_ENABLED */\n");
         export.write("#endif /* LIBXML_REGEXP_ENABLED */\n");
         output.write("#endif /* LIBXML_REGEXP_ENABLED */\n");
+    elif file == "xmlschemas" or file == "xmlschemastypes" or \
+         file == "relaxng":
+        include.write("#endif /* LIBXML_SCHEMAS_ENABLED */\n");
+        export.write("#endif /* LIBXML_SCHEMAS_ENABLED */\n");
+        output.write("#endif /* LIBXML_SCHEMAS_ENABLED */\n");
     return 1
 
 def buildStubs():
@@ -531,6 +553,7 @@ def buildStubs():
 #    wrapper.write("#include \"config.h\"\n")
     wrapper.write("#include <libxml/xmlversion.h>\n")
     wrapper.write("#include <libxml/tree.h>\n")
+    wrapper.write("#include <libxml/xmlschemastypes.h>\n")
     wrapper.write("#include \"libxml_wrap.h\"\n")
     wrapper.write("#include \"libxml2-py.h\"\n\n")
     for function in functions.keys():
@@ -597,6 +620,11 @@ classes_type = {
     "xmlOutputBufferPtr": ("._o", "outputBuffer(_obj=%s)", "outputBuffer"),
     "xmlParserInputBufferPtr": ("._o", "inputBuffer(_obj=%s)", "inputBuffer"),
     "xmlRegexpPtr": ("._o", "xmlReg(_obj=%s)", "xmlReg"),
+    "xmlTextReaderLocatorPtr": ("._o", "xmlTextReaderLocator(_obj=%s)", "xmlTextReaderLocator"),
+    "xmlTextReaderPtr": ("._o", "xmlTextReader(_obj=%s)", "xmlTextReader"),
+    'xmlRelaxNGPtr': ('._o', "relaxNgSchema(_obj=%s)", "relaxNgSchema"),
+    'xmlRelaxNGParserCtxtPtr': ('._o', "relaxNgParserCtxt(_obj=%s)", "relaxNgParserCtxt"),
+    'xmlRelaxNGValidCtxtPtr': ('._o', "relaxNgValidCtxt(_obj=%s)", "relaxNgValidCtxt"),
 }
 
 converter_type = {
@@ -616,6 +644,8 @@ classes_ancestor = {
     "xmlAttribute" : "xmlNode",
     "outputBuffer": "ioWriteWrapper",
     "inputBuffer": "ioReadWrapper",
+    "parserCtxt": "parserCtxtCore",
+    "xmlTextReader": "xmlTextReaderCore",
 }
 classes_destructors = {
     "parserCtxt": "xmlFreeParserCtxt",
@@ -624,11 +654,21 @@ classes_destructors = {
 #    "outputBuffer": "xmlOutputBufferClose",
     "inputBuffer": "xmlFreeParserInputBuffer",
     "xmlReg": "xmlRegFreeRegexp",
+    "xmlTextReader": "xmlFreeTextReader",
+    "relaxNgSchema": "xmlRelaxNGFree",
+    "relaxNgParserCtxt": "xmlRelaxNGFreeParserCtxt",
+    "relaxNgValidCtxt": "xmlRelaxNGFreeValidCtxt",
 }
 
 functions_noexcept = {
     "xmlHasProp": 1,
     "xmlHasNsProp": 1,
+    "xmlDocSetRootElement": 1,
+}
+
+reference_keepers = {
+    "xmlTextReader": [('inputBuffer', 'input')],
+    "relaxNgValidCtxt": [('relaxNgSchema', 'schema')],
 }
 
 function_classes = {}
@@ -676,6 +716,10 @@ def nameFixup(name, classe, type, file):
         func = "regexp" + name[9:]
     elif name[0:6] == "xmlReg" and file == "xmlregexp":
         func = "regexp" + name[6:]
+    elif name[0:20] == "xmlTextReaderLocator" and file == "xmlreader":
+        func = name[20:]
+    elif name[0:13] == "xmlTextReader" and file == "xmlreader":
+        func = name[13:]
     elif name[0:11] == "xmlACatalog":
         func = name[11:]
         func = string.lower(func[0:1]) + func[1:]
@@ -825,8 +869,6 @@ def buildWrappers():
 		func = nameFixup(name, classe, type, file)
 		info = (1, func, name, ret, args, file)
 		function_classes[classe].append(info)
-	    if found == 1:
-		break
 	if found == 1:
 	    continue
 	if name[0:8] == "xmlXPath":
@@ -926,6 +968,10 @@ def buildWrappers():
 		classes.write("class %s(%s):\n" % (classname,
 			      classes_ancestor[classname]))
 		classes.write("    def __init__(self, _obj=None):\n")
+		if reference_keepers.has_key(classname):
+		    rlist = reference_keepers[classname]
+		    for ref in rlist:
+		        classes.write("        self.%s = None\n" % ref[1])
 		classes.write("        self._o = None\n")
 		classes.write("        %s.__init__(self, _obj=_obj)\n\n" % (
 			      classes_ancestor[classname]))
@@ -939,6 +985,10 @@ def buildWrappers():
 		txt.write("Class %s()\n" % (classname))
 		classes.write("class %s:\n" % (classname))
 		classes.write("    def __init__(self, _obj=None):\n")
+		if reference_keepers.has_key(classname):
+		    list = reference_keepers[classname]
+		    for ref in list:
+		        classes.write("        self.%s = None\n" % ref[1])
 		classes.write("        if _obj != None:self._o = _obj;return\n")
 		classes.write("        self._o = None\n\n");
 	    if classes_destructors.has_key(classname):
@@ -1024,9 +1074,30 @@ def buildWrappers():
 			    classes.write(
 		    "        if ret is None:raise treeError('%s() failed')\n"
 					  % (name))
-			classes.write("        return ");
+
+			#
+			# generate the returned class wrapper for the object
+			#
+			classes.write("        __tmp = ");
 			classes.write(classes_type[ret[0]][1] % ("ret"));
 			classes.write("\n");
+
+                        #
+			# Sometime one need to keep references of the source
+			# class in the returned class object.
+			# See reference_keepers for the list
+			#
+			tclass = classes_type[ret[0]][2]
+			if reference_keepers.has_key(tclass):
+			    list = reference_keepers[tclass]
+			    for pref in list:
+				if pref[0] == classname:
+				    classes.write("        __tmp.%s = self\n" %
+						  pref[1])
+			#
+			# return the class
+			#
+			classes.write("        return __tmp\n");
 		    elif converter_type.has_key(ret[0]):
 			#
 			# Raise an exception
