@@ -35,6 +35,7 @@
 #include <libxml/DOCBparser.h>
 #include <libxml/tree.h>
 #include <libxml/debugXML.h>
+#include <libxml/globals.h>
 
 #ifdef LIBXML_DEBUG_ENABLED
 static int debug = 0;
@@ -74,7 +75,8 @@ xmlSAXHandler emptySAXHandlerStruct = {
     NULL, /* xmlParserError */
     NULL, /* getParameterEntity */
     NULL, /* cdataBlock */
-    NULL  /* externalSubset */
+    NULL, /* externalSubset */
+    1
 };
 
 xmlSAXHandlerPtr emptySAXHandler = &emptySAXHandlerStruct;
@@ -95,7 +97,7 @@ extern xmlSAXHandlerPtr debugSAXHandler;
  * Returns 1 if true
  */
 static int
-isStandaloneDebug(void *ctx ATTRIBUTE_UNUSED)
+isStandaloneDebug(void *ctxt ATTRIBUTE_UNUSED)
 {
     fprintf(stdout, "SAX.isStandalone()\n");
     return(0);
@@ -110,7 +112,7 @@ isStandaloneDebug(void *ctx ATTRIBUTE_UNUSED)
  * Returns 1 if true
  */
 static int
-hasInternalSubsetDebug(void *ctx ATTRIBUTE_UNUSED)
+hasInternalSubsetDebug(void *ctxt ATTRIBUTE_UNUSED)
 {
     fprintf(stdout, "SAX.hasInternalSubset()\n");
     return(0);
@@ -125,20 +127,20 @@ hasInternalSubsetDebug(void *ctx ATTRIBUTE_UNUSED)
  * Returns 1 if true
  */
 static int
-hasExternalSubsetDebug(void *ctx ATTRIBUTE_UNUSED)
+hasExternalSubsetDebug(void *ctxt ATTRIBUTE_UNUSED)
 {
     fprintf(stdout, "SAX.hasExternalSubset()\n");
     return(0);
 }
 
 /**
- * hasInternalSubsetDebug:
+ * internalSubsetDebug:
  * @ctxt:  An XML parser context
  *
  * Does this document has an internal subset
  */
 static void
-internalSubsetDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name,
+internalSubsetDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name,
 	       const xmlChar *ExternalID, const xmlChar *SystemID)
 {
     fprintf(stdout, "SAX.internalSubset(%s,", name);
@@ -167,7 +169,7 @@ internalSubsetDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name,
  * Returns the xmlParserInputPtr if inlined or NULL for DOM behaviour.
  */
 static xmlParserInputPtr
-resolveEntityDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *publicId, const xmlChar *systemId)
+resolveEntityDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *publicId, const xmlChar *systemId)
 {
     /* xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx; */
 
@@ -199,7 +201,7 @@ resolveEntityDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *publicId, const xm
  * Returns the xmlParserInputPtr if inlined or NULL for DOM behaviour.
  */
 static xmlEntityPtr
-getEntityDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name)
+getEntityDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name)
 {
     fprintf(stdout, "SAX.getEntity(%s)\n", name);
     return(NULL);
@@ -215,7 +217,7 @@ getEntityDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name)
  * Returns the xmlParserInputPtr
  */
 static xmlEntityPtr
-getParameterEntityDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name)
+getParameterEntityDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name)
 {
     fprintf(stdout, "SAX.getParameterEntity(%s)\n", name);
     return(NULL);
@@ -234,7 +236,7 @@ getParameterEntityDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name)
  * An entity definition has been parsed
  */
 static void
-entityDeclDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name, int type,
+entityDeclDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name, int type,
           const xmlChar *publicId, const xmlChar *systemId, xmlChar *content)
 {
     fprintf(stdout, "SAX.entityDecl(%s, %d, %s, %s, %s)\n",
@@ -250,7 +252,7 @@ entityDeclDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name, int type,
  * An attribute definition has been parsed
  */
 static void
-attributeDeclDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *elem,
+attributeDeclDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *elem,
 	           const xmlChar *name, int type, int def,
 		   const xmlChar *defaultValue,
 	           xmlEnumerationPtr tree ATTRIBUTE_UNUSED)
@@ -269,7 +271,7 @@ attributeDeclDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *elem,
  * An element definition has been parsed
  */
 static void
-elementDeclDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name, int type,
+elementDeclDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name, int type,
 	         xmlElementContentPtr content ATTRIBUTE_UNUSED)
 {
     fprintf(stdout, "SAX.elementDecl(%s, %d, ...)\n",
@@ -286,7 +288,7 @@ elementDeclDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name, int type,
  * What to do when a notation declaration has been parsed.
  */
 static void
-notationDeclDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name,
+notationDeclDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name,
 	     const xmlChar *publicId, const xmlChar *systemId)
 {
     fprintf(stdout, "SAX.notationDecl(%s, %s, %s)\n",
@@ -304,7 +306,7 @@ notationDeclDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name,
  * What to do when an unparsed entity declaration is parsed
  */
 static void
-unparsedEntityDeclDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name,
+unparsedEntityDeclDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name,
 		   const xmlChar *publicId, const xmlChar *systemId,
 		   const xmlChar *notationName)
 {
@@ -322,7 +324,7 @@ unparsedEntityDeclDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name,
  * Everything is available on the context, so this is useless in our case.
  */
 static void
-setDocumentLocatorDebug(void *ctx ATTRIBUTE_UNUSED,
+setDocumentLocatorDebug(void *ctxt ATTRIBUTE_UNUSED,
 	                xmlSAXLocatorPtr loc ATTRIBUTE_UNUSED)
 {
     fprintf(stdout, "SAX.setDocumentLocator()\n");
@@ -335,7 +337,7 @@ setDocumentLocatorDebug(void *ctx ATTRIBUTE_UNUSED,
  * called when the document start being processed.
  */
 static void
-startDocumentDebug(void *ctx ATTRIBUTE_UNUSED)
+startDocumentDebug(void *ctxt ATTRIBUTE_UNUSED)
 {
     fprintf(stdout, "SAX.startDocument()\n");
 }
@@ -347,7 +349,7 @@ startDocumentDebug(void *ctx ATTRIBUTE_UNUSED)
  * called when the document end has been detected.
  */
 static void
-endDocumentDebug(void *ctx ATTRIBUTE_UNUSED)
+endDocumentDebug(void *ctxt ATTRIBUTE_UNUSED)
 {
     fprintf(stdout, "SAX.endDocument()\n");
 }
@@ -360,7 +362,7 @@ endDocumentDebug(void *ctx ATTRIBUTE_UNUSED)
  * called when an opening tag has been processed.
  */
 static void
-startElementDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name, const xmlChar **atts)
+startElementDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name, const xmlChar **atts)
 {
     int i;
 
@@ -394,7 +396,7 @@ startElementDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name, const xmlChar
  * called when the end of an element has been detected.
  */
 static void
-endElementDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name)
+endElementDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name)
 {
     fprintf(stdout, "SAX.endElement(%s)\n", (char *) name);
 }
@@ -409,7 +411,7 @@ endElementDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name)
  * Question: how much at a time ???
  */
 static void
-charactersDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *ch, int len)
+charactersDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *ch, int len)
 {
     unsigned char output[40];
     int inlen = len, outlen = 30;
@@ -428,7 +430,7 @@ charactersDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *ch, int len)
  * called when an entity reference is detected. 
  */
 static void
-referenceDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name)
+referenceDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name)
 {
     fprintf(stdout, "SAX.reference(%s)\n", name);
 }
@@ -444,7 +446,7 @@ referenceDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name)
  * Question: how much at a time ???
  */
 static void
-ignorableWhitespaceDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *ch, int len)
+ignorableWhitespaceDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *ch, int len)
 {
     char output[40];
     int i;
@@ -466,7 +468,7 @@ ignorableWhitespaceDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *ch, int len)
  * A processing instruction has been parsed.
  */
 static void
-processingInstructionDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *target,
+processingInstructionDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *target,
                       const xmlChar *data)
 {
     fprintf(stdout, "SAX.processingInstruction(%s, %s)\n",
@@ -481,7 +483,7 @@ processingInstructionDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *target,
  * A comment has been parsed.
  */
 static void
-commentDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *value)
+commentDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *value)
 {
     fprintf(stdout, "SAX.comment(%s)\n", value);
 }
@@ -495,7 +497,7 @@ commentDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *value)
  * called when a pcdata block has been parsed
  */
 static void
-cdataBlockDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *value, int len)
+cdataBlockDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *value, int len)
 {
     fprintf(stdout, "SAX.pcdata(%.20s, %d)\n",
 	    (char *) value, len);
@@ -508,7 +510,7 @@ cdataBlockDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *value, int len)
  * Does this document has an external subset
  */
 static void
-externalSubsetDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name,
+externalSubsetDebug(void *ctxt ATTRIBUTE_UNUSED, const xmlChar *name,
 	       const xmlChar *ExternalID, const xmlChar *SystemID)
 {
     fprintf(stdout, "SAX.externalSubset(%s,", name);
@@ -532,7 +534,7 @@ externalSubsetDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name,
  * extra parameters.
  */
 static void
-warningDebug(void *ctx ATTRIBUTE_UNUSED, const char *msg, ...)
+warningDebug(void *ctxt ATTRIBUTE_UNUSED, const char *msg, ...)
 {
     va_list args;
 
@@ -552,7 +554,7 @@ warningDebug(void *ctx ATTRIBUTE_UNUSED, const char *msg, ...)
  * extra parameters.
  */
 static void
-errorDebug(void *ctx ATTRIBUTE_UNUSED, const char *msg, ...)
+errorDebug(void *ctxt ATTRIBUTE_UNUSED, const char *msg, ...)
 {
     va_list args;
 
@@ -572,7 +574,7 @@ errorDebug(void *ctx ATTRIBUTE_UNUSED, const char *msg, ...)
  * extra parameters.
  */
 static void
-fatalErrorDebug(void *ctx ATTRIBUTE_UNUSED, const char *msg, ...)
+fatalErrorDebug(void *ctxt ATTRIBUTE_UNUSED, const char *msg, ...)
 {
     va_list args;
 
@@ -609,7 +611,8 @@ xmlSAXHandler debugSAXHandlerStruct = {
     fatalErrorDebug,
     getParameterEntityDebug,
     cdataBlockDebug,
-    externalSubsetDebug
+    externalSubsetDebug,
+    1
 };
 
 xmlSAXHandlerPtr debugSAXHandler = &debugSAXHandlerStruct;
@@ -640,7 +643,7 @@ parseSAXFile(char *filename) {
 	    res = fread(chars, 1, 4, f);
 	    if (res > 0) {
 		ctxt = docbCreatePushParserCtxt(emptySAXHandler, NULL,
-			    chars, res, filename, 0);
+			    chars, res, filename, XML_CHAR_ENCODING_NONE);
 		while ((res = fread(chars, 1, size, f)) > 0) {
 		    docbParseChunk(ctxt, chars, res, 0);
 		}
@@ -666,7 +669,7 @@ parseSAXFile(char *filename) {
 		res = fread(chars, 1, 4, f);
 		if (res > 0) {
 		    ctxt = docbCreatePushParserCtxt(debugSAXHandler, NULL,
-				chars, res, filename, 0);
+				chars, res, filename, XML_CHAR_ENCODING_NONE);
 		    while ((res = fread(chars, 1, size, f)) > 0) {
 			docbParseChunk(ctxt, chars, res, 0);
 		    }
@@ -722,7 +725,7 @@ parseAndPrintFile(char *filename) {
 	    res = fread(chars, 1, 4, f);
 	    if (res > 0) {
 		ctxt = docbCreatePushParserCtxt(NULL, NULL,
-			    chars, res, filename, 0);
+			    chars, res, filename, XML_CHAR_ENCODING_NONE);
 		while ((res = fread(chars, 1, size, f)) > 0) {
 		    docbParseChunk(ctxt, chars, res, 0);
 		}
